@@ -3,13 +3,20 @@
 """
 
 import numpy as np
-
-__all__ = ['trap_grad', 'spiral_varden', 'spiral_arch', 'epi', 'rosette',
-           'stack_of', 'traj_array_to_complex', 'traj_complex_to_array']
+from scipy import interpolate
+from scipy import integrate
+import numba as nb
+import math
 
 
-def trap_grad(area, gmax, dgdt, dt, *args):
-    r"""General trapezoidal gradient designer. Min total time.
+__all__ = ['min_trap_grad', 'trap_grad', 'spiral_varden', 'spiral_arch', 'epi',
+           'rosette', 'spokes_grad', 'stack_of', 'traj_array_to_complex',
+           'traj_complex_to_array', 'min_time_gradient']
+
+
+def min_trap_grad(area, gmax, dgdt, dt):
+    r"""Minimal duration trapezoidal gradient designer. Design for target area
+    under the flat portion (for non-ramp-sampled pulses)
 
     Args:
         area (float): pulse area in (g*sec)/cm
@@ -17,6 +24,58 @@ def trap_grad(area, gmax, dgdt, dt, *args):
         dgdt (float): max slew rate in g/cm/sec
         dt (float): sample time in sec
 
+    Returns:
+        2-element tuple containing
+
+        - **trap** (*array*): gradient waveform in g/cm.
+        - **ramppts** (*int*): number of points in ramps.
+
+    """
+
+    if np.abs(area) > 0:
+        # we get the solution for plateau amp by setting derivative of
+        # duration as a function of amplitude to zero and solving
+        a = np.sqrt(dgdt * area / 2)
+
+        # finish design with discretization
+        # make a flat portion of magnitude a and enough area for the swath
+        pts = np.floor(area / a / dt)
+        flat = np.ones((1, int(pts)))
+        flat = flat / np.sum(flat) * area / dt
+        if np.max(flat) > gmax:
+            flat = np.ones((1, int(np.ceil(area / gmax / dt))))
+            flat = flat / np.sum(flat) * area / dt
+
+        # make attack and decay ramps
+        ramppts = int(np.ceil(np.max(flat) / dgdt / dt))
+        ramp_up = np.linspace(0, ramppts, num=ramppts+1) / ramppts*np.max(flat)
+        ramp_dn = np.linspace(ramppts, 0, num=ramppts+1) / ramppts*np.max(flat)
+
+        trap = np.concatenate((ramp_up, np.squeeze(flat), ramp_dn))
+
+    else:
+        # negative-area trap requested?
+        trap, ramppts = 0, 0
+
+    return np.expand_dims(trap, axis=0), ramppts
+
+
+def trap_grad(area, gmax, dgdt, dt, *args):
+    r"""General trapezoidal gradient designer for total target area
+    (for rewinders)
+
+    Args:
+        area (float): pulse area in (g*sec)/cm
+        gmax (float): maximum gradient in g/cm
+        dgdt (float): max slew rate in g/cm/sec
+        dt (float): sample time in sec
+
+    Returns:
+        2-element tuple containing
+
+        - **trap** (*array*): gradient waveform in g/cm.
+        - **ramppts** (*int*): number of points in ramps.
+
     """
 
     if len(args) < 5:
@@ -26,21 +85,21 @@ def trap_grad(area, gmax, dgdt, dt, *args):
     if np.abs(area) > 0:
         if rampsamp:
 
-            ramppts = np.ceil(gmax/dgdt/dt)
+            ramppts = int(np.ceil(gmax/dgdt/dt))
             triareamax = ramppts * dt * gmax
 
             if triareamax > np.abs(area):
                 # triangle pulse
                 newgmax = np.sqrt(np.abs(area) * dgdt)
                 ramppts = int(np.ceil(newgmax/dgdt/dt))
-                ramp_up = np.linspace(0, ramppts, num=ramppts)/ramppts
-                ramp_dn = np.linspace(ramppts, 0, num=ramppts)/ramppts
+                ramp_up = np.linspace(0, ramppts, num=ramppts+1)/ramppts
+                ramp_dn = np.linspace(ramppts, 0, num=ramppts+1)/ramppts
                 pulse = np.concatenate((ramp_up, ramp_dn))
             else:
                 # trapezoid pulse
                 nflat = int(np.ceil((area - triareamax)/gmax / dt / 2) * 2)
-                ramp_up = np.linspace(0, ramppts, num=ramppts) / ramppts
-                ramp_dn = np.linspace(ramppts, 0, num=ramppts) / ramppts
+                ramp_up = np.linspace(0, ramppts, num=ramppts+1) / ramppts
+                ramp_dn = np.linspace(ramppts, 0, num=ramppts+1) / ramppts
                 pulse = np.concatenate((ramp_up, np.ones(nflat), ramp_dn))
 
             trap = pulse * (area / (sum(pulse) * dt))
@@ -54,8 +113,8 @@ def trap_grad(area, gmax, dgdt, dt, *args):
 
             # make attack and decay ramps
             ramppts = int(np.ceil(np.max(flat) / dgdt / dt))
-            ramp_up = np.linspace(0, ramppts, num=ramppts) / ramppts * flat_top
-            ramp_dn = np.linspace(ramppts, 0, num=ramppts) / ramppts * flat_top
+            ramp_up = np.linspace(0, ramppts, num=ramppts+1) / ramppts*flat_top
+            ramp_dn = np.linspace(ramppts, 0, num=ramppts+1) / ramppts*flat_top
             trap = np.concatenate((ramp_up, flat, ramp_dn))
 
     else:
@@ -81,6 +140,15 @@ def spiral_varden(fov, res, gts, gslew, gamp, densamp, dentrans, nl,
         nl (float): degree of undersampling outer region.
         rewinder (Boolean): if True, include rewinder. If false, exclude.
 
+    Returns:
+        tuple: (g, k, t, s, dens) tuple containing
+
+        - **g** - (array): gradient waveform [g/cm]
+        - **k** - (array): exact k-space corresponding to gradient g.
+        - **time** - (array):  sampled time
+        - **s** - (array): slew rate [g/cm/s]
+        - **dens** - (array): undersampling factor at each time point.
+
     References:
         Code and algorithm based on spiralgradlx6 from
         Doug Noll, U. of Michigan BME
@@ -286,6 +354,14 @@ def spiral_arch(fov, res, gts, gslew, gamp):
         gslew (float): max slew rate in mT/m/ms.
         gamp (float): max gradient amplitude in mT/m.
 
+    Returns:
+        tuple: (g, k, t, s) tuple containing
+
+        - **g** - (array): gradient waveform [mT/m]
+        - **k** - (array): exact k-space corresponding to gradient g.
+        - **time** - (array):  sampled time
+        - **s** - (array): slew rate [mT/m/ms]
+
     References:
         Glover, G. H.(1999).
         Simple Analytic Spiral K-Space Algorithm.
@@ -370,6 +446,14 @@ def epi(fov, n, etl, dt, gamp, gslew, offset=0, dirx=-1, diry=1):
         dirx (int): x direction of EPI -1 left to right, 1 right to left
         diry (int): y direction of EPI -1 bottom-top, 1 top-bottom
 
+    Returns:
+        tuple: (g, k, t, s) tuple containing
+
+        - **g** - (array): gradient waveform [mT/m]
+        - **k** - (array): exact k-space corresponding to gradient g.
+        - **time** - (array):  sampled time
+        - **s** - (array): slew rate [mT/m/ms]
+
 
     References:
         From Antonis Matakos' contrib to Jeff Fessler's IRT.
@@ -510,6 +594,14 @@ def rosette(kmax, w1, w2, dt, dur, gamp=None, gslew=None):
         gamp (float): max gradient amplitude (mT/m).
         gslew (float): max slew rate (mT/m/ms).
 
+    Returns:
+        tuple: (g, k, t, s) tuple containing
+
+        - **g** - (array): gradient waveform [mT/m]
+        - **k** - (array): exact k-space corresponding to gradient g.
+        - **time** - (array):  sampled time
+        - **s** - (array): slew rate [mT/m/ms]
+
     References:
         D. C. Noll, 'Multi-shot rosette trajectories for spectrally selective
         MR imaging.' IEEE Trans. Med Imaging 16, 372-377 (1997).
@@ -545,6 +637,70 @@ def rosette(kmax, w1, w2, dt, dur, gamp=None, gslew=None):
     return g, k, t, s
 
 
+def spokes_grad(k, tbw, sl_thick, gmax, dgdtmax, gts):
+    r""" Spokes gradient designer. Given some chosen spoke locations k, return
+    the gradients required to move between those spoke locations.
+
+    Args:
+        k (array): spokes locations, [Nspokes, 2]
+        tbw (int): time bandwidth product.
+        sl_thick (float): slice thickness (mm).
+        gmax (float): max gradient amplitude (g/cm).
+        dgdtmax (float): max gradient slew (g/cm/s).
+        gts (float): hardware sampling dwell time (s).
+
+    Returns:
+        g (array): gz, gy, and gz waveforms  in g/cm [3, Nt]
+
+    References:
+           Grissom, W., Khalighi, M., Sacolick, L., Rutt, B. & Vogel, M (2012).
+           Small-tip-angle spokes pulse design using interleaved greedy and
+           local optimization methods. Magnetic Resonance in Medicine, 68(5),
+           1553-62.
+
+    """
+    n_spokes = k.shape[0]
+
+    area = tbw / (sl_thick / 10) / 4257  # thick * kwid = twb, kwid = gam*area
+    [subgz, nramp] = min_trap_grad(area, gmax, dgdtmax, gts)
+
+    # calc gradient, add extra 0 location at end for return to (0, 0)
+    gxarea = np.diff(np.concatenate((k[:, 0], np.zeros(1)))) / 4257
+    gyarea = np.diff(np.concatenate((k[:, 1], np.zeros(1)))) / 4257
+
+    gx, gy, gz = [], [], []
+    gz_sign = -1
+    for ii in range(n_spokes):
+        gz_sign *= -1
+        gz.extend(np.squeeze(gz_sign * subgz).tolist())  # alt sign of gz
+
+        gx.extend([0] * np.size(subgz))  # zeros for gz duration
+        if np.absolute(gxarea[ii]) > 0:
+            [gblip, _] = trap_grad(abs(gxarea[ii]), gmax, dgdtmax, gts)
+            gxblip = np.int(np.sign(gxarea[ii])) * gblip
+            gx = gx[:len(gx) - len(gxblip.T)]
+            gx.extend(np.squeeze(gxblip).tolist())
+
+        gy.extend([0] * np.size(subgz))
+        if np.absolute(gyarea[ii]) > 0:
+            [gblip, _] = trap_grad(abs(gyarea[ii]), gmax, dgdtmax, gts)
+            gyblip = np.int(np.sign(gyarea[ii])) * gblip
+            gy = gy[:len(gy) - len(gyblip.T)]
+            gy.extend(np.squeeze(gyblip).tolist())
+
+    [gref, _] = trap_grad(gts * np.sum(subgz) / 2, gmax, dgdtmax, gts)
+    gzref = - gref
+    gz.extend(np.squeeze(gzref).tolist())
+    gx.extend([0] * np.size(gzref))
+    gy.extend([0] * np.size(gzref))
+
+    # combine gradient waveforms
+    gx = np.array(gx)
+    g = np.vstack((np.array(gx), np.array(gy), np.array(gz)))
+
+    return g
+
+
 def stack_of(k, num, zres):
     r"""Function for creating a 3D stack of ____ trajectory from a 2D [Nt 2]
     trajectory.
@@ -590,3 +746,227 @@ def traj_array_to_complex(k):
     """
     kout = k[:, 0] + 1j * k[:, 1]
     return kout
+
+
+@nb.jit(nopython=True, cache=True)  # pragma: no cover
+def runge_kutta(ds: float, st: float, kvals: np.ndarray, smax=None,
+                gamma=4.257):
+    r"""Runge-Kutta 4 for curve constrained
+
+    Args:
+        ds (float): spacing in arc length space
+        st (float): output shape.
+        kvals (array): 3 points of curve.
+        smax (float): maximum slew
+        gamma (float): gyromagnetic ratio
+
+    Returns:
+        float or None: step size dsdt or None
+    """
+    temp = (gamma ** 2 * smax ** 2 - abs(kvals[0]) ** 2 * st ** 4)
+    if temp < 0.0:
+        return None
+    k1 = ds / st * math.sqrt(temp)
+
+    temp = \
+        (gamma ** 2 * smax ** 2 - abs(kvals[1]) ** 2 * (st + ds * k1 / 2) ** 4)
+    if temp < 0.0:
+        return None
+    k2 = ds / (st + ds * k1 / 2) * math.sqrt(temp)
+
+    temp = \
+        (gamma ** 2 * smax ** 2 - abs(kvals[1]) ** 2 * (st + ds * k2 / 2) ** 4)
+    if temp < 0.0:
+        return None
+    k3 = ds / (st + ds * k2 / 2) * math.sqrt(temp)
+
+    temp = \
+        (gamma ** 2 * smax ** 2 - abs(kvals[2]) ** 2 * (st + ds * k3) ** 4)
+    if temp < 0.0:
+        return None
+    k4 = ds / (st + ds * k3) * math.sqrt(temp)
+
+    return k1 / 6 + k2 / 3 + k3 / 3 + k4 / 6
+
+
+#  Arc length code translated from matlab
+#    (c) Michael Lustig 2005
+#    modified 2006 and 2007
+#    Rewritten in Python in 2020 by Kevin Johnson
+def min_time_gradient(c: np.ndarray, g0=0, gfin=0, gmax=4, smax=15,
+                      dt=4e-3, gamma=4.257):
+    r"""
+    Given a k-space trajectory c(n), gradient and slew constraints. This
+    function will return a new parametrization that will meet these
+    constraint while getting from one point to the other in minimum time.
+
+    Args:
+        c (array): Curve in k-space given in any parametrization [1/cm]
+                        Nx3 real array
+        g0 (float): Initial gradient amplitude (leave empty for g0 = 0)
+        gfin (float): Gradient value at the end of the trajectory. If not
+                        possible, the result would be the largest possible
+                        ampltude. (Leave empty if you don't care to get
+                        maximum gradient.)
+        gmax (float): Maximum gradient [G/cm] (3.9 default)
+        smax (float): Maximum slew [G/Cm/ms]  (14.5 default)
+        dt (float): Sampling time interval [ms] (4e-3 default)
+        gamma (float): Gyromagnetic ratio
+
+    Returns:
+        tuple: (g, k, s, t) tuple containing
+
+        - **g** - (array): gradient waveform [G/cm]
+        - **k** - (array): exact k-space corresponding to gradient g.
+        - **s** - (array): slew rate [G/cm/ms]
+        - **time** - (array):  sampled time
+
+    References:
+        Lustig M, Kim SJ, Pauly JM. A fast method for designing time-optimal
+        gradient waveforms for arbitrary k-space trajectories. IEEE Trans Med
+        Imaging. 2008;27(6):866-873. doi:10.1109/TMI.2008.922699
+    """
+
+    def sdotmax(cs: interpolate.CubicSpline, s: np.ndarray,
+                gmax, smax, gamma=4.257):
+        # [sdot, k, ] = sdotMax(PP, p_of_s, s, gmax, smax)
+        #
+        # Given a k-space curve C (in [1/cm] units), maximum gradient amplitude
+        # (in G/cm) and maximum slew-rate (in G/(cm*ms)).
+        # This function calculates the upper bound for the time parametrization
+        # sdot (which is a non scaled max gradient constaint) as a function
+        # of s.
+        #
+        #   cs      --  spline polynomial
+        #   p_of_s  --  parametrization vs arclength
+        #   s       --  arclength parametrization (0->1)
+        #   gmax    --  maximum gradient (G/cm)
+        #   smax    --  maximum slew rate (G/ cm*ms)
+        #
+        #   returns the maximum sdot (1st derivative of s) as a function of
+        #   arclength s
+        #   Also, returns curvature as a function of s and length of curve (L)
+        #
+        #  (c) Michael Lustig 2005
+        #  last modified 2006
+
+        # Absolute value of 2nd derivative in curve space using cubic splines
+        cs2 = cs.derivative(2)  # spline derivative
+        cs2_highres = cs2(s)  # evaluated along arc length
+        k = np.linalg.norm(cs2_highres, axis=1)  # magnitude
+
+        # calc I constraint curve (maximum gradient)
+        sdot1 = gamma * gmax * np.ones_like(s)
+
+        # calc II constraint curve (curve curvature dependent)
+        sdot2 = np.sqrt(gamma * smax / (k + np.finfo(float).eps))
+
+        # calc total constraint
+        sdot = np.minimum(sdot1, sdot2)
+
+        return sdot, k
+
+    # Curve in arbitrary paramater space, cubic spline
+    num_p = c.shape[0]
+    p = np.linspace(0, 1, num_p, endpoint=True)
+    cp = interpolate.CubicSpline(p, c, axis=0)
+
+    # Integrate absolute value to find length and s(arc) vs p(paramater)
+    cp1_spline = cp.derivative()
+    p_highres = np.linspace(0, 1, num_p * 10)
+    cp1_highres = cp1_spline(p_highres)
+    ds_p = np.linalg.norm(cp1_highres, axis=1)
+
+    # s vs p to enable conversion
+    s_of_p = integrate.cumtrapz(ds_p, p_highres, initial=0)
+    curve_length = s_of_p[-1]
+
+    # decide ds and compute st for the first point
+    stt0 = (gamma * smax)  # always assumes first point is max slew
+    st0 = stt0 * dt / 8  # start at 1/8 the gradient for accuracy close to g=0
+    s0 = st0 * dt
+    ds = s0 / 4.0  # smaller step size for numerical accuracy
+    ns = int(curve_length / ds)
+
+    if g0 is None:
+        g0 = 0
+
+    # s is arc length at high resolution
+    s = np.linspace(0, curve_length, ns, endpoint=True)
+
+    # Cubic spline at s positions (s of p)
+    cp_highres = cp(p_highres)
+    cs = interpolate.CubicSpline(s_of_p, cp_highres, axis=0)
+
+    # compute constraints (forbidden line curve)
+    phi, k = sdotmax(cs, s, gmax, smax)
+
+    # extend for the Runge-Kutte method
+    k = np.pad(k, (0, 3), 'constant', constant_values=(0,))
+
+    # Get the start
+    sta = np.zeros_like(s)
+    sta[0] = min(g0 * gamma + st0, gamma * gmax)
+
+    # solve ODE forward
+    for n in range(1, s.shape[0]):
+        kpos = n
+        dstds = runge_kutta(ds, sta[n - 1], k[kpos:kpos + 4], smax)
+
+        if dstds is None:
+            sta[n] = phi[n]
+        else:
+            tmpst = sta[n - 1] + dstds
+            sta[n] = min(tmpst, phi[n])
+
+    stb = 0 * s
+    if gfin is None:
+        stb[-1] = sta[-1]
+    else:
+        stb[-1] = min(max(gfin * gamma, st0), gamma * gmax)
+
+    # solve ODE backwards
+    for n in range(s.shape[0] - 2, 0, -1):
+
+        kpos_end = n  # to 0
+        kpos = kpos_end + 3
+        dstds = runge_kutta(ds, stb[n + 1], k[kpos:(kpos - 3):-1], smax)
+
+        if dstds is None:
+            stb[n] = phi[n - 1]
+        else:
+            tmpst = stb[n + 1] + dstds
+            stb[n] = min(tmpst, phi[n - 1])
+
+    # Fix last point which is indexed a bit off
+    n = 0
+    kpos_end = n
+    kpos = kpos_end + 3
+    dstds = runge_kutta(ds, stb[n + 1], k[kpos::-1], smax)
+    if dstds is None:
+        stb[n] = phi[n * 2 - 1]
+    else:
+        tmpst = stb[n + 1] + dstds
+        stb[n] = min(tmpst, phi[n - 1])
+
+    # take the minimum of the curves
+    ds = s[1] - s[0]
+    st_of_s = np.minimum(sta, stb)
+
+    # compute time
+    t_of_s = integrate.cumtrapz(1. / st_of_s, initial=0) * ds
+
+    t = np.arange(0, t_of_s[-1] + np.finfo(float).eps, dt)
+
+    t_of_s = interpolate.CubicSpline(t_of_s, s)
+    s_of_t = t_of_s(t)
+    c = np.squeeze(cs(s_of_t))
+
+    g = np.diff(c, axis=0, append=np.zeros((1, 3))) / gamma / dt
+    g[-1, :] = g[-2, :] + g[-2, :] - g[-3, :]
+
+    k = integrate.cumtrapz(g, t, initial=0, axis=0) * gamma
+
+    s = np.diff(g, axis=0) / dt
+
+    return g, k, s, t
